@@ -1,21 +1,21 @@
-import numpy
+from __future__ import absolute_import, print_function, division
+import numpy as np
 
 import theano
 
 
 def test_detect_nan():
-    """
-    Test the code snippet example that detects NaN values.
-    """
+    # Test the code snippet example that detects NaN values.
+
     nan_detected = [False]
 
     def detect_nan(i, node, fn):
         for output in fn.outputs:
-            if numpy.isnan(output[0]).any():
-                print '*** NaN detected ***'
+            if np.isnan(output[0]).any():
+                print('*** NaN detected ***')
                 theano.printing.debugprint(node)
-                print 'Inputs : %s' % [input[0] for input in fn.inputs]
-                print 'Outputs: %s' % [output[0] for output in fn.outputs]
+                print('Inputs : %s' % [input[0] for input in fn.inputs])
+                print('Outputs: %s' % [output[0] for output in fn.outputs])
                 nan_detected[0] = True
                 break
 
@@ -28,18 +28,17 @@ def test_detect_nan():
 
 
 def test_optimizer():
-    """
-    Test that we can remove optimizer
-    """
+    # Test that we can remove optimizer
+
     nan_detected = [False]
 
     def detect_nan(i, node, fn):
         for output in fn.outputs:
-            if numpy.isnan(output[0]).any():
-                print '*** NaN detected ***'
+            if np.isnan(output[0]).any():
+                print('*** NaN detected ***')
                 theano.printing.debugprint(node)
-                print 'Inputs : %s' % [input[0] for input in fn.inputs]
-                print 'Outputs: %s' % [output[0] for output in fn.outputs]
+                print('Inputs : %s' % [input[0] for input in fn.inputs])
+                print('Outputs: %s' % [output[0] for output in fn.outputs])
                 nan_detected[0] = True
                 break
 
@@ -57,24 +56,23 @@ def test_optimizer():
 
 
 def test_not_inplace():
-    """
-    Test that we can remove optimizers including inplace optimizers
-    """
+    # Test that we can remove optimizers including inplace optimizers
+
     nan_detected = [False]
 
     def detect_nan(i, node, fn):
         for output in fn.outputs:
-            if numpy.isnan(output[0]).any():
-                print '*** NaN detected ***'
+            if np.isnan(output[0]).any():
+                print('*** NaN detected ***')
                 theano.printing.debugprint(node)
-                print 'Inputs : %s' % [input[0] for input in fn.inputs]
-                print 'Outputs: %s' % [output[0] for output in fn.outputs]
+                print('Inputs : %s' % [input[0] for input in fn.inputs])
+                print('Outputs: %s' % [output[0] for output in fn.outputs])
                 nan_detected[0] = True
                 break
 
     x = theano.tensor.vector('x')
     mode = theano.compile.MonitorMode(post_func=detect_nan)
-    #mode = mode.excluding('fusion', 'inplace')
+    # mode = mode.excluding('fusion', 'inplace')
     mode = mode.excluding('local_elemwise_fusion',
                           'inplace_elemwise_optimizer')
     o = theano.tensor.outer(x, x)
