@@ -1,8 +1,9 @@
+from __future__ import absolute_import, print_function, division
 #
 # Slice type and Op. None Type and NoneConst.
 #
 
-import numpy
+import numpy as np
 
 import theano
 from theano.gof import Apply, Constant, Generic, Op, Type, hashtype
@@ -15,12 +16,15 @@ def as_int_none_variable(x):
     elif NoneConst.equals(x):
         return x
     x = theano.tensor.as_tensor_variable(x, ndim=0)
-    if x.type.dtype[:3] not in ('int', 'uin'):
+    if x.type.dtype not in theano.tensor.integer_dtypes:
         raise TypeError('index must be integers')
     return x
 
 
 class MakeSlice(Op):
+
+    __props__ = ()
+
     def make_node(self, slc, stop=None, step=None):
         # We need to accept and handle in make_node inputs the node
         # inputs to allow redoing a new op elsewhere in the graph by
@@ -32,22 +36,13 @@ class MakeSlice(Op):
         else:
             inp = [slc, stop, step]
         return Apply(self,
-                     map(as_int_none_variable, inp),
+                     list(map(as_int_none_variable, inp)),
                      [slicetype()])
 
     def perform(self, node, inp, out_):
         out, = out_
         out[0] = slice(*inp)
 
-    def __str__(self):
-        return self.__class__.__name__
-
-    def __eq__(self, other):
-        return type(self) == type(other)
-
-    def __hash__(self):
-        return hash(type(self))
-
     def grad(self, inputs, grads):
         return [DisconnectedType()() for i in inputs]
 
@@ -83,17 +78,17 @@ class SliceConstant(Constant):
     def __init__(self, type, data, name=None):
         assert isinstance(data, slice)
         # Numpy ndarray aren't hashable, so get rid of them.
-        if isinstance(data.start, numpy.ndarray):
+        if isinstance(data.start, np.ndarray):
             assert data.start.ndim == 0
-            assert "int" in str(data.start.dtype)
+            assert str(data.start.dtype) in theano.tensor.integer_dtypes
             data = slice(int(data.start), data.stop, data.step)
-        elif isinstance(data.stop, numpy.ndarray):
+        elif isinstance(data.stop, np.ndarray):
             assert data.stop.ndim == 0
-            assert "int" in str(data.stop.dtype)
+            assert str(data.stop.dtype) in theano.tensor.integer_dtypes
             data = slice(data.start, int(data.stop), data.step)
-        elif isinstance(data.step, numpy.ndarray):
+        elif isinstance(data.step, np.ndarray):
             assert data.step.ndim == 0
-            assert "int" in str(data.step.dtype)
+            assert str(data.step.dtype) in theano.tensor.integer_dtypes
             data = slice(data.start, int(data.stop), data.step)
         Constant.__init__(self, type, data, name)
 
@@ -111,6 +106,7 @@ SliceType.Constant = SliceConstant
 class NoneTypeT(Generic):
     """
     Inherit from Generic to have c code working.
+
     """
 
     def filter(self, x, strict=False, allow_downcast=None):
